package di_aop.aop.not_null;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;

public class NotNullValidationIntercepter implements MethodInterceptor {

	public Object invoke(MethodInvocation invocation) throws Throwable {
		Object[] arguments = invocation.getArguments();
		Method method = invocation.getMethod();
		String methodName = method.getName();
		Class<?>[] parameterTypes = method.getParameterTypes();

		validate(method, arguments);

		Class<?>[] interfaces = method.getDeclaringClass().getInterfaces();
		for (Class<?> clazz : interfaces) {
			try {
				method = clazz.getDeclaredMethod(methodName, parameterTypes);
				validate(method, arguments);
			} catch (NoSuchMethodException ignore) {
			}
		}

		return invocation.proceed();
	}

	private void validate(Method method, Object[] arguments) {
		Annotation[][] parameterAnnotations = method.getParameterAnnotations();
		for (int i = 0; i < parameterAnnotations.length; i++) {
			Annotation[] annotations = parameterAnnotations[i];
			for (Annotation annotation : annotations) {
				if (annotation instanceof NotNull == false) {
					continue;
				}
				NotNull notNull = (NotNull) annotation;
				if (notNull != null && arguments[i] == null) {
					throw new IllegalArgumentException(
							"arg was null.[arg index=" + i + "]");
				}
			}
		}
	}

}